#lang racket

(require vigracket/config)
(require vigracket/helpers)
(require scheme/foreign)
(unsafe!)


(define-cpointer-type _camera-capture)
  
;#################Get the capture pointer from opencv ###########
(define opencv_get_cam_capture_c
    (get-ffi-obj 'opencv_get_cam_capture_c dylib-path
                (_fun [device : _int]
                      -> [capture_ptr : _camera-capture])))

(define opencv_release_cam_capture_c
  (get-ffi-obj 'opencv_release_cam_capture_c dylib-path
                (_fun [capture_ptr : _camera-capture]
                      -> _void)))
  
(define opencv_get_cam_frame_width_ptr_c
    (get-ffi-obj 'opencv_get_cam_frame_width_ptr_c dylib-path
                (_fun [capture_ptr : _camera-capture]
                      -> [width : _int])))

(define opencv_get_cam_frame_height_ptr_c
    (get-ffi-obj 'opencv_get_cam_frame_height_ptr_c dylib-path
                (_fun [capture_ptr : _camera-capture]
                      -> [height : _int])))

(define opencv_get_cam_fps_ptr_c
    (get-ffi-obj 'opencv_get_cam_fps_ptr_c dylib-path
                (_fun [capture_ptr : _camera-capture]
                      -> [height : _int])))

(define opencv_get_cam_frame_count_ptr_c
    (get-ffi-obj 'opencv_get_cam_frame_count_ptr_c dylib-path
                (_fun [capture_ptr : _camera-capture]
                      -> [height : _int])))

(define opencv_grab_camera_rgbimage_ptr_c
  (get-ffi-obj 'opencv_grab_camera_rgbimage_ptr_c dylib-path
               (_fun (img_vector_r img_vector_g img_vector_b width height device) :: [img_vector_r : _cvector]
                     [img_vector_g : _cvector]
                     [img_vector_b : _cvector]
                     [width : _int]
                     [height : _int]
                     [device : _camera-capture]
                     -> (res :  _int))))

(define (start-camera-capture device_id)
  (opencv_get_cam_capture_c device_id))

(define (stop-camera-capture device_handle)
  (begin
    (opencv_release_cam_capture_c device_handle)
    (set-cpointer-tag! device_handle  "invalid")))

(define (grabimage device_handle)
  (if (equal?  (cpointer-tag device_handle) "camera-capture")
      (let* ((width (opencv_get_cam_frame_width_ptr_c   device_handle))
             (height (opencv_get_cam_frame_height_ptr_c device_handle))
             (img (make-image width height 3 0.0 0.0 0.0))
             (foo  (opencv_grab_camera_rgbimage_ptr_c (image-data img 0)  (image-data img 1)  (image-data img 2) width height device_handle)))
        (case foo
          ((0) img)
          ((1) (error "Error in vigracket.grab.grabimage: Image cannot be grabbed by opencv!"))
          ((2) (error "Error in vigracket.grab.grabimage: Image is not RGB colored"))
          ((3) (error "Error in vigracket.grab.grabimage: Sizes do not match!"))))
      (error "Error in vigracket.grab.grabimage: incompatible capture used!")))
  
  
(define image-grab grabimage)

(provide    grabimage
            image-grab
            start-camera-capture
            stop-camera-capture)